﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Programmable {
  [Item("Encoding", "Base class for describing different encodings.")]
  [StorableClass]
  public abstract class Encoding<T> : NamedItem, IEncoding
    where T : class,ISolutionCreator {
    public override sealed bool CanChangeName {
      get { return false; }
    }

    public virtual IEnumerable<IValueParameter> Parameters {
      get { return Enumerable.Empty<IValueParameter>(); }
    }

    protected List<IOperator> encodingOperators = new List<IOperator>();
    [Storable]
    public IEnumerable<IOperator> Operators {
      get { return encodingOperators; }
      protected set { encodingOperators = new List<IOperator>(value); }
    }


    ISolutionCreator IEncoding.DefaultSolutionCreator { get { return DefaultSolutionCreator; } }
    public virtual T DefaultSolutionCreator {
      get { return null; }
    }


    ISolutionCreator IEncoding.SolutionCreator {
      get { return SolutionCreator; }
      set {
        if (!(value is T)) throw new ArgumentException("???");
        SolutionCreator = (T)value;
      }
    }
    private T solutionCreator;
    public T SolutionCreator {
      get {
        if (solutionCreator == null) return DefaultSolutionCreator;
        return solutionCreator;
      }
      set {
        if (value == null) throw new ArgumentNullException("SolutionCreator must not be null.");
        if (solutionCreator == value) return;
        solutionCreator = value;
        OnSolutionCreatorChanged();
      }
    }

    [StorableConstructor]
    protected Encoding(bool deserializing) : base(deserializing) { }

    protected Encoding(Encoding<T> original, Cloner cloner)
      : base(original, cloner) {
      encodingOperators = original.Operators.Select(cloner.Clone).ToList();
      solutionCreator = cloner.Clone(original.solutionCreator);
    }
    protected Encoding(string name) : base(name) { }



    public void ConfigureOperator(IOperator @operator) {
      ConfigureOperators(new[] { @operator });
    }
    public virtual void ConfigureOperators(IEnumerable<IOperator> operators) {

    }

    protected virtual void OnSolutionCreatorChanged() {
      ConfigureOperator(SolutionCreator);
    }


    public event EventHandler ParameterConfigurationChanged;
    protected virtual void OnParameterConfigurationChanged() {
      var handler = ParameterConfigurationChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }
  }
}
